/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


#ifndef __DRV_GFX3D_VFY_H
#define __DRV_GFX3D_VFY_H

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
//#include "x_graphics3d.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

//the below two macros must be oppisite
//either load data via ICE or load date via USB Host
#define T32_HOST     0
#define USB_G3D_VFY  1


/* some pattern such as:
F000_read_writer_reg 
F000-MMU-APB 
F013-MMU_GP_IRQ
need disable to pass */

/* register test pattern check INT status reg  in mali600, need disable TEST_G3D_INTERRUPT to pass  */
#define TEST_G3D_INTERRUPT (0)

#define NON_CACHEABLE_DRAM_OFFSET (0x0)
#define TEST_DRAM_CHB (0)   



//Mali600 IRQ number
#define VECTOR_MALI600_IRQJOB  21
#define VECTOR_MALI600_IRQMMU  22
#define VECTOR_MALI600_IRQGPU  23


#if T32_HOST
enum
{
	E_T32_NOP,
	E_T32_EXEC,
	E_T32_RESTART,
	E_T32_EXIT,
	E_T32_EXEC_RELOAD_SYM,
	E_T32_DETECT_HOST,
	E_T32_END
};
extern INT32 T32_HostExec(INT32 i4TargetFlag, const CHAR *szFormat, ...);
#endif



typedef struct
{
  INT32 handle;
  INT32 offset;
  UINT32 filesize;
}
FILE;

/* fixed build warnings */
#if 0
typedef enum 
{
    G3D_CMP_STATUS_UNKNOW = 0,
    G3D_CMP_STATUS_OPEN_FAIL =1,                        
    G3D_CMP_STATUS_SUCCESS = 2,                        
    G3D_CMP_STATUS_FAIL = 4,                       
    G3D_CMP_STATUS_MAX = 32,
} G3D_CMP_STATUS;
#else
#define G3D_CMP_STATUS_UNKNOW       (0)
#define G3D_CMP_STATUS_OPEN_FAIL    (1)						
#define G3D_CMP_STATUS_SUCCESS      (2) 					   
#define G3D_CMP_STATUS_FAIL         (4)					   
#define G3D_CMP_STATUS_MAX          (32)
typedef UINT32 G3D_CMP_STATUS;
#endif

typedef struct
{
    BOOL  fgIsMaliBaseTest;
	BOOL  fgIsChannelBpattern;
    BOOL  fgIsAxiBrdgWROutStand;
    BOOL  fgIsGPWatchDogDisable;
    BOOL  fgIsPPWatchDogDisable;
    BOOL  fgIsCompareAllMem;
    BOOL  fgIsCompareGPStatus;
    BOOL  fgIsComparePPStatus;
    BOOL  fgIsAxiStopTest;
    UINT32  u4ReadStand;
    UINT32  u4WriteStand;
} G3D_EMU_CONFIG_T;

typedef struct
{    
    G3D_CMP_STATUS  eCmpMemResult;
    G3D_CMP_STATUS  eCmpRegResult;
    
    G3D_CMP_STATUS  eCmpMemPass;
    G3D_CMP_STATUS  eCmpRegPass;
    
    BOOL                      fgIsGpIrqDone;
    BOOL                      fgIsPpIrqDone;
} G3D_EMU_RESULT_T;


/// Init Graphics3D driver.
/// Call this function by system initialization function when system boot up.
/// \return If return value < 0, it's failed.
extern INT32 i4Graphics3D_Drv_Init(void);
extern INT32 Gfx3dEmuDisplay(void);

G3D_CMP_STATUS G3DSLTResult(void);

/// Uninit Graphics3D driver.
/// Call this function by system uninitialization function when system shut down.
/// \return If return value < 0, it's failed.
extern INT32 i4Graphics3D_Drv_Uninit(void);

/// Send Stop Command to Cmd Buffer.
/// It's FF Command, use to Stop current command and clean the cmd buffer.
/// \return If return value < 0, it's failed.
extern INT32 i4Graphics3D_Drv_Stop(
    UINT32              u4CmdBufHandle);

extern INT32 i4Graphics3D_Drv_Emu(void);

extern INT32 i4Graphics3D_Drv_RegRW(void);

#endif // __DRV_GFX3D_VFY_H
