/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: dtvbm11 $
 * $Date: 2015/11/10 $
 * $RCSfile: osd_if.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file osd_if.h
 *  This header file declares public function prototypes of OSD.
 */

#ifndef OSD_IF_H
#define OSD_IF_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_lint.h"
LINT_EXT_HEADER_BEGIN
#include "x_typedef.h"
LINT_EXT_HEADER_END

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------

/** OSD plane enum.
 */

typedef enum
{
    OSD_PLANE_1, OSD_PLANE_2, OSD_PLANE_3,
#ifdef OSD_PLA4_SUPPORT
    OSD_PLANE_4, //scart out
#endif
#ifdef OSD_PLA5_SUPPORT
    OSD_PLANE_5, //cursor
#endif
    OSD_PLANE_MAX_NUM
} OSD_PLANE_T;

/** OSD scaler enum.
 */
typedef enum
{
    OSD_SCALER_2,
    OSD_SCALER_3,
    OSD_SCALER_1,
    OSD_SCALER_MAX_NUM
} OSD_SCALER_T;

/** OSD function return code.
 */
typedef enum
{
    OSD_RET_OK,
    OSD_RET_INV_ARG,
    OSD_RET_OUT_OF_MEM,
    OSD_RET_OUT_OF_REGION,
    OSD_RET_OUT_OF_LIST,
    OSD_RET_UNINIT,
    OSD_RET_INV_PLANE,
    OSD_RET_INV_SCALER,
    OSD_RET_INV_REGION,
    OSD_RET_INV_BITMAP,
    OSD_RET_INV_LIST,
    OSD_RET_INV_DISPLAY_MODE,
    OSD_RET_REGION_COLLISION,
    OSD_RET_ERR_INTERNAL,
    OSD_RET_UNDEF_ERR
} OSD_RET_CODE_T;

/** OSD color mode.
 */
typedef enum
{
    OSD_CM_YCBCR_CLUT2,
    OSD_CM_YCBCR_CLUT4,
    OSD_CM_YCBCR_CLUT8,
    OSD_CM_RESERVED_0,
    OSD_CM_CBYCRY422_DIRECT16,
    OSD_CM_YCBYCR422_DIRECT16,
    OSD_CM_AYCBCR8888_DIRECT32,
    OSD_CM_RESERVED_1,
    OSD_CM_RGB_CLUT2,
    OSD_CM_RGB_CLUT4,
    OSD_CM_RGB_CLUT8,
    OSD_CM_RGB565_DIRECT16,
    OSD_CM_ARGB1555_DIRECT16,
    OSD_CM_ARGB4444_DIRECT16,
    OSD_CM_ARGB8888_DIRECT32,
    OSD_CM_ALPHA_BIT
} OSD_COLOR_MODE_T;

/** OSD blending mode.
 */
typedef enum
{
    OSD_BM_NONE, OSD_BM_PIXEL, OSD_BM_REGION, OSD_BM_PLANE
} OSD_BLEND_MODE_T;

/** OSD region cmd.
 */
typedef enum
{
    OSD_RGN_ALLOC,
    OSD_RGN_PREV,
    OSD_RGN_NEXT,
    OSD_RGN_FLAGS,
    OSD_RGN_POS_X,
    OSD_RGN_POS_Y,
    OSD_RGN_BMP_W,
    OSD_RGN_BMP_H,
    OSD_RGN_DISP_W,
    OSD_RGN_DISP_H,
    OSD_RGN_OUT_W,
    OSD_RGN_OUT_H,
    OSD_RGN_COLORMODE,
    OSD_RGN_ALPHA,
    OSD_RGN_BMP_ADDR,
    OSD_RGN_BMP_PITCH,
    OSD_RGN_CLIP_V,
    OSD_RGN_CLIP_H,
    OSD_RGN_V_FLIP,
    OSD_RGN_H_MIRROR,
    OSD_RGN_PAL_LOAD,
    OSD_RGN_PAL_ADDR,
    OSD_RGN_PAL_LEN,
    OSD_RGN_STEP_V,
    OSD_RGN_STEP_H,
    OSD_RGN_COLOR_KEY,
    OSD_RGN_COLOR_KEY_EN,
    OSD_RGN_MIX_SEL,
    OSD_RGN_BIG_ENDIAN,
    OSD_RGN_ALPHA_SEL,
    OSD_RGN_YR_SEL,
    OSD_RGN_UG_SEL,
    OSD_RGN_VB_SEL,
    OSD_RGN_NEXT_EN,
    OSD_RGN_NEXT_HDR_ADDR,
    OSD_RGN_FIFO_EX,
    OSD_RGN_HPX_EXT,
    OSD_RGN_ENDIAN,
    OSD_RGN_HINTP_MODE,
    OSD_RGN_VINTP_MODE,
    OSD_RGN_HSTEP_OFFSET,
    OSD_RGN_VSTEP_OFFSET,
    OSD_RGN_DEC_MODE,
    OSD_RGN_DEC_LINE,
    OSD_RGN_DEC_EN
} OSD_RGN_CMD_T;

//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

/** input color mode, width and get coresspoding pitch size */
#define OSD_GET_PITCH_SIZE(CM, WIDTH, SIZE) \
    do \
    { \
        switch ((CM)) \
        { \
        case OSD_CM_CBYCRY422_DIRECT16: \
        case OSD_CM_YCBYCR422_DIRECT16: \
        case OSD_CM_RGB565_DIRECT16: \
        case OSD_CM_ARGB1555_DIRECT16: \
        case OSD_CM_ARGB4444_DIRECT16: \
            (SIZE) = (WIDTH) * 2; \
            break; \
        case OSD_CM_AYCBCR8888_DIRECT32: \
        case OSD_CM_ARGB8888_DIRECT32: \
            (SIZE) = (WIDTH) * 4; \
            break; \
        case OSD_CM_YCBCR_CLUT8: \
        case OSD_CM_RGB_CLUT8: \
            (SIZE) = (WIDTH); \
            break; \
        case OSD_CM_YCBCR_CLUT4: \
        case OSD_CM_RGB_CLUT4: \
            (SIZE) = (WIDTH) >> 1; \
            break; \
        case OSD_CM_YCBCR_CLUT2: \
        case OSD_CM_RGB_CLUT2: \
            (SIZE) = (WIDTH) >> 2; \
            break; \
        default: \
            ASSERT(0); \
        } \
    } while (0)

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

/** reset osd driver and hardware status
 *  @param  void
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_Reset(void);

EXTERN INT32 OSD_SwReset(void);
/** set osd plane position
 *  @param  u4Plane plane id
 *  @param  u4X x position
 *  @param  u4Y y position
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_BASE_SetOsdPosition(UINT32 u4Plane, UINT32 u4X, UINT32 u4Y);

/** get osd plane position
 *  @param  u4Plane plane id
 *  @param  pu4X x position pointer
 *  @param  pu4Y y position pointer
 *  @retval OSD_RET_OK success
 */
EXTERN INT32
        OSD_BASE_GetOsdPosition(UINT32 u4Plane, UINT32* pu4X, UINT32* pu4Y);

/** enable/disable osd plane, it will also disable scaler when disable plane
 *  @param  u4Plane plane id
 *  @param  fgEnable enable=TRUE/disable=FALSE
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_Enable(UINT32 u4Plane, BOOL fgEnable);

/** attach a specific list to osd plane
 *  @param  u4Plane plane id
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_FlipTo(UINT32 u4Plane, UINT32 u4RgnList);

/** get osd plane alpha level
 *  @param  u4Plane plane id
 *  @param  pu1BlendLevel aplha level pointer
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_GetBlendLevel(UINT32 u4Plane, UINT8* pu1BlendLevel);

/** set osd plane alpha level
 *  @param  u4Plane plane id
 *  @param  u1BlendLevel aplha level(0~255)
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_SetBlendLevel(UINT32 u4Plane, UINT8 u1BlendLevel);

/** get osd plane fading value
 *  @param  u4Plane plane id
 *  @param  pu1Fading fading value pointer
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_GetFading(UINT32 u4Plane, UINT8* pu1Fading);

/** set osd plane fading value
 *  @param  u4Plane plane id
 *  @param  u1Fading fading value
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_SetFading(UINT32 u4Plane, UINT8 u1Fading);

/** dump osd debug info
 *  @param  u4Plane osd plane
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_PLA_Dump(UINT32 u4Plane);

/** get the first region id of osd plane
 *  @param  u4Plane osd plane
 *  @retval region id, -1 is no region
 */
EXTERN INT32 OSD_PLA_GetFirstRegion(UINT32 u4Plane);

/** get the first region address of osd plane
 *  @param  u4Plane osd plane
 *  @retval region address, 0 is no region
 */
EXTERN UINT32 OSD_PLA_GetFirstRegionAddress(UINT32 u4Plane);

/** create an osd list
 *  @param pu4RgnList list id pointer
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_RGN_LIST_Create(UINT32 * pu4RgnList);

/** delete an osd list
 *  @param  u4RgnList list id
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_RGN_LIST_Delete(UINT32 u4RgnList);

/** get first region id of list
 *  @param  u4RgnList list id
 *  @param  pi4HeadRegion region id pointer
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_RGN_LIST_GetHead(UINT32 u4RgnList, INT32 * phHeadRegion);

/** get osd region attribute
 *  @param  u4Region region id
 *  @param  i4Cmd attribute name, refer to OSD_RGN_CMD_T
 *  @param  pu4Value attribute value pointer
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_RGN_Get(UINT32 u4Region, INT32 i4Cmd, UINT32 * pu4Value);

/** set osd region attribute
 *  @param  u4Region region id
 *  @param  i4Cmd attribute name, refer to OSD_RGN_CMD_T
 *  @param  u4Value attribute value
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_RGN_Set(UINT32 u4Region, INT32 i4Cmd, UINT32 u4Value);

/** create an osd region
 *  @param  pu4Region region id pointer
 *  @param  u4BmpWidth source bitmap width
 *  @param  u4BmpHeight source bitmap height
 *  @param  pvBitmap source bitmap address, MUST align with 16bytes
 *  @param  eColorMode, region color mode, refer to OSD_COLOR_MODE_T
 *  @param  u4BmpPitch, bytes per line, MUST align with 16bytes
 *  @param  u4DispX, region x position
 *  @param  u4DispY, region y position
 *  @param  u4DispW, output width
 *  @param  u4DispH, output height

 *  @retval OSD_RET_OK success
 *  @retval OSD_RET_INV_ARG invalid argument
 */
EXTERN INT32 OSD_RGN_Create(UINT32 * pu4Region, UINT32 u4BmpWidth,
        UINT32 u4BmpHeight, void * pvBitmap, UINT32 eColorMode,
        UINT32 u4BmpPitch, UINT32 u4DispX, UINT32 u4DispY, UINT32 u4DispW,
        UINT32 u4DispH);

/** delete an osd region
 *  @param  u4Region region id
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_RGN_Delete(UINT32 u4Region);

/** attach region to list
 *  @param  u4Region region id
 *  @param  u4RgnList list id
 *  @retval OSD_RET_OK success
 *  @retval OSD_RET_INV_LIST invalid list
 */
EXTERN INT32 OSD_RGN_Insert(UINT32 u4Region, UINT32 u4RgnList);

/** dettach region from list
 *  @param  u4Region region id
 *  @param  u4RgnList list id
 *  @retval OSD_RET_OK success
 *  @retval OSD_RET_INV_REGION invalid region
 */
EXTERN INT32 OSD_RGN_Detach(UINT32 u4Region, UINT32 u4RgnList);

EXTERN INT32 OSD_RGN_Copy(UINT32 u4DstRgn, UINT32 u4SrcRgn);
/** cinfugure osd scaler
 *  @param  u4Scaler scaler id(only OSD_SCALER_2)
 *  @param  fgEnable enable=TRUE/disable=FALSE
 *  @param  u4SrcWidth osd input width
 *  @param  u4SrcHeight osd input height
 *  @param  u4DstWidth osd output width
 *  @param  u4DstHeight osd output height
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_SC_Scale(UINT32 u4Scaler, UINT32 u4Enable, UINT32 u4SrcWidth,
        UINT32 u4SrcHeight, UINT32 u4DstWidth, UINT32 u4DstHeight);

/** get osd scaler infomation
 *  @param  u4Scaler scaler id(only OSD_SCALER_2)
 *  @param  pu4Enable enable pointer
 *  @param  pu4SrcW osd input width pointer
 *  @param  pu4SrcH osd input height pointer
 *  @param  pu4DstW osd output width pointer
 *  @param  pu4DstH osd output height pointer
 *  @param  pu4Is16Bpp don't care
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_SC_GetScalerInfo(UINT32 u4Scaler, UINT32* pu4Enable,
        UINT32* pu4SrcW, UINT32* pu4SrcH, UINT32* pu4DstW, UINT32* pu4DstH,
        UINT32* pu4Is16Bpp);

/** enable low pass filter
 *  @param  u4Scaler scaler id(only OSD_SCALER_2)
 *  @param  fgEnable enable=TRUE/disable=FALSE
 *  @retval OSD_RET_OK success
 */
EXTERN INT32 OSD_SC_SetLpf(UINT32 u4Scaler, UINT32 u4Enable);

EXTERN INT32 OSD_SetTVEOut(BOOL fgEnable);
EXTERN INT32 OSD_SetPowerDwn(BOOL fgEnable);
EXTERN INT32 OSD_ShowLogo(void);

EXTERN INT32 OSD_PLA_Set3DRegion(UINT32 u4Plane, UINT32 u4LeftRgn,
        UINT32 u4RightRgn);
EXTERN INT32 OSD_BASE_EnableAutoLR(UINT32 u4Enable);
EXTERN INT32 OSD_BASE_GetLRStatus(UINT32* pu4Status);

EXTERN INT32 OSD_BASE_SetLineShift(UINT32 u4Enable, UINT32 u4Shift);

EXTERN INT32 OSD_BASE_SetPlaneSwitch(UINT32 u4Enable);
EXTERN INT32 OSD_BASE_SetLineDup(UINT32 u4Enable);
EXTERN INT32 OSD_BASE_SetPlaneLineShift(UINT32 u4Plane, BOOL fgEnable,
        INT32 i4Value);
EXTERN INT32 OSD_BASE_Set3DShutterLineShift(UINT32 u4Enable, UINT32 u4Shift);
EXTERN INT32 OSD_BASE_Set3DPrLineShift(UINT32 u4Enable, UINT32 u4Shift);
EXTERN INT32 OSD_BASE_Get3DOsdShiftEn(UINT32 OSD_Plane,UINT32 *pu4Value);
EXTERN INT32 OSD_BASE_Get3DOsd2LShiftEn(UINT32 *pu4Value);

EXTERN BOOL OSD_BASE_Get_3DOSD2PLsEn(void);
EXTERN void OSD_BASE_Set_3DOSD2PLsEn(BOOL u4Enable);
EXTERN INT32 OSD_3D_SetOsd2LShiftEn(UINT32 u4Value);
EXTERN INT32 OSD_3D_SetOsd1LShiftEn(UINT32 u4Value);
EXTERN INT32 OSD_3D_SetOsd3LShiftEn(UINT32 u4Value);
EXTERN BOOL OSD_BASE_IsDisplaying(void);

#ifdef CC_ANDROID_3D_UI_SUPPORT
EXTERN UINT32 OSD_PLA_SetUIMode(UINT32 u4Mode, UINT32 u4WIdth, UINT32 u4Height);
#endif

#ifdef CC_TDTV_3D_OSD_ENABLE
EXTERN INT32 OSD_BASE_Set3DStatus(void);
EXTERN INT32 OSD_BASE_Enabe3DShutter( UINT32 u4Enable, UINT32 u4Shift);
#endif

EXTERN INT32 OSD_PLA_SetPremulEn(UINT32 u4Plane, UINT32 u4Value);
EXTERN INT32 OSD_PLA_GetPremulEn(UINT32 u4Plane, UINT32 *pu4Value);
EXTERN INT32 OSD_BASE_SetScrnHStart(UINT32 u4_plane,UINT32 u4_value);
EXTERN INT32 OSD_BASE_SetScrnVStartTop(UINT32 u4_value);
EXTERN INT32 OSD_BASE_UpdateHwReg(void);
EXTERN void OSD_PLA_WaitStable(UINT32 u4_plane);
EXTERN void OSD_WaitVsync(void);
EXTERN void OSD_WaitVsync2(void);

EXTERN INT32 OSD_SetFb3DCfg(UINT32 u4_plane, UINT32 u4_3d_mode, UINT32 u4_sync);

#endif //OSD_IF_H
